/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */
package org.apache.causeway.core.config.metamodel.facets;

import org.apache.causeway.applib.annotation.CollectionLayout;
import org.apache.causeway.applib.annotation.PropertyLayout;
import org.apache.causeway.core.config.CausewayConfiguration;

/**
 * Enums shared for config of {@link PropertyLayout} and {@link CollectionLayout}.
 */
public final class AssociationLayoutConfigOptions {

    public enum SequencePolicy {
        AS_PER_SEQUENCE,
        ALPHABETICALLY;

        public String toNameLower() {
            return name().toLowerCase();
        }

        public static SequencePolicy forProperty(final CausewayConfiguration configuration) {
            return configuration.getApplib().getAnnotation().getPropertyLayout()
                    .getSequencePolicyIfUnreferenced();
        }

        public static SequencePolicy forCollection(final CausewayConfiguration configuration) {
            return configuration.getApplib().getAnnotation().getCollectionLayout()
                    .getSequencePolicyIfUnreferenced();
        }
    }

}
